export async function up(entrances, arg = ['*'], ref) {
    const { includes, excludes } = Array.isArray(arg)
        ? { includes: arg, excludes: [] }
        : arg;
    const includeMatchers = includes.map(include => buildMatcher(include));
    const excludeMatchers = excludes.map(exclude => buildMatcher(exclude));
    const entryPromises = [];
    for (const key in entrances) {
        if (!includeMatchers.some(matcher => matcher(key)) ||
            excludeMatchers.some(matcher => matcher(key))) {
            continue;
        }
        entryPromises.push(Promise.resolve(entrances[key]).then(value => [
            key,
            {
                get() {
                    return value;
                },
            },
        ]));
    }
    if (ref) {
        Object.setPrototypeOf(ref, entrances);
        for (const [key, descriptor] of await Promise.all(entryPromises)) {
            Object.defineProperty(ref, key, descriptor);
        }
        return ref;
    }
    else {
        return Object.create(entrances, Object.fromEntries(await Promise.all(entryPromises)));
    }
}
function buildMatcher(pattern) {
    const starIndex = pattern.indexOf('*');
    if (starIndex < 0) {
        return key => key === pattern;
    }
    else {
        const prefix = pattern.slice(0, starIndex);
        const postfix = pattern.slice(starIndex + 1);
        return key => key.startsWith(prefix) && key.endsWith(postfix);
    }
}
//# sourceMappingURL=up.js.map